@@ -16,7 +16,7 @@
 %                              December 2001                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright @ 2001 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -63,6 +63,9 @@
 #include "MagickCore/thread-private.h"
 #include "MagickCore/utility.h"
 #include "MagickCore/utility-private.h"
+#if defined(MAGICKCORE_HAVE_GETENTROPY)
+#include <sys/random.h>
+#endif
 /*
   Define declarations.
 */
@@ -346,7 +349,7 @@ static ssize_t ReadRandom(int file,unsigned char *source,size_t length)
 
 static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
 {
-#define MaxEntropyExtent  64
+#define MaxEntropyExtent  64  /* max permitted: 256 */
 
   MagickThreadType
     tid;
@@ -367,6 +370,20 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
   */
   entropy=AcquireStringInfo(0);
   LockSemaphoreInfo(random_info->semaphore);
+#if defined(MAGICKCORE_HAVE_GETENTROPY)
+  {
+    int
+      status;
+    
+    SetStringInfoLength(entropy,MaxEntropyExtent);
+    status=getentropy(GetStringInfoDatum(entropy),MaxEntropyExtent);
+    if (status == 0)
+      {
+        UnlockSemaphoreInfo(random_info->semaphore);
+        return(entropy);
+      }
+  }
+#endif
   chaos=AcquireStringInfo(sizeof(unsigned char *));
   SetStringInfoDatum(chaos,(unsigned char *) &entropy);
   ConcatenateStringInfo(entropy,chaos);
@@ -413,7 +430,7 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
     if (gettimeofday(&timer,(struct timezone *) NULL) == 0)
       {
         seconds=(size_t) timer.tv_sec;
-        nanoseconds=((size_t) 1000UL*timer.tv_usec);
+        nanoseconds=(size_t) (1000UL*timer.tv_usec);
       }
   }
 #endif
