@@ -3,23 +3,21 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%                     IIIIIIIIII    PPPPPPPP      LL                          %
-%                         II        PP      PP    LL                          %
-%                         II        PP       PP   LL                          %
-%                         II        PP      PP    LL                          %
-%                         II        PPPPPPPP      LL                          %
-%                         II        PP            LL                          %
-%                         II        PP            LL                          %
-%                     IIIIIIIIII    PP            LLLLLLLL                    %
+%                             IIIII  PPPP   L                                 %
+%                               I    P   P  L                                 %
+%                               I    PPPP   L                                 %
+%                               I    P      L                                 %
+%                             IIIII  P      LLLLL                             %
 %                                                                             %
 %                                                                             %
+%                 Read/Write Scanalytics IPLab Image Format                   %
 %                                                                             %
-%                   Read/Write Scanalytics IPLab Image Format                 %
-%                                  Sean Burke                                 %
-%                                  2008.05.07                                 %
-%                                     v 0.9                                   %
+%                              Software Design                                %
+%                                Sean Burke                                   %
+%                                 May 2008                                    %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%                                                                             %
+%  Copyright @ 2008 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,33 +38,32 @@
 
 /*
  Include declarations.
- */
-#include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/cache.h"
-#include "magick/colorspace.h"
-#include "magick/colorspace-private.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/option.h"
-#include "magick/pixel-accessor.h"
-#include "magick/property.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/module.h"
-
-/*
-  Typedef declarations.
 */
+#include "MagickCore/studio.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/colorspace.h"
+#include "MagickCore/colorspace-private.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/option.h"
+#include "MagickCore/property.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/module.h"
+
+/* 
+Tyedef declarations
+ */
 
 typedef struct _IPLInfo
 {
@@ -83,7 +80,7 @@ typedef struct _IPLInfo
 } IPLInfo;
 
 static MagickBooleanType
-  WriteIPLImage(const ImageInfo *,Image *);
+  WriteIPLImage(const ImageInfo *,Image *,ExceptionInfo *);
 
 /*
 static void increase (void *pixel, int byteType){
@@ -93,10 +90,10 @@ static void increase (void *pixel, int byteType){
     case 2:(*((unsigned int *) pixel))++; break;
     case 3:(*((signed long *) pixel))++; break;
     default:(*((unsigned int *) pixel))++; break;
-  }
+  }  
 }
 */
-
+
 /*
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %                                                                             %
@@ -130,7 +127,7 @@ static MagickBooleanType IsIPL(const unsigned char *magick,const size_t length)
     return(MagickTrue);
   return(MagickFalse);
 }
-
+
 /*
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %                                                                             %
@@ -142,13 +139,13 @@ static MagickBooleanType IsIPL(const unsigned char *magick,const size_t length)
  %                                                                             %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %
- %  ReadIPLImage() reads a Scanalytics IPLab image file and returns it.  It
- %  allocates the memory necessary for the new Image structure and returns a
+ %  ReadIPLImage() reads a Scanalytics IPLab image file and returns it.  It 
+ %  allocates the memory necessary for the new Image structure and returns a 
  %  pointer to the new image.
  %
  %  According to the IPLab spec, the data is blocked out in five dimensions:
  %  { t, z, c, y, x }.  When we return the image, the latter three are folded
- %  into the standard "Image" structure.  The "scenes" (image_info->scene)
+ %  into the standard "Image" structure.  The "scenes" (image_info->scene) 
  %  correspond to the order: { {t0,z0}, {t0, z1}, ..., {t1,z0}, {t1,z1}... }
  %  The number of scenes is t*z.
  %
@@ -160,7 +157,7 @@ static MagickBooleanType IsIPL(const unsigned char *magick,const size_t length)
  %
  %    o image_info: The image info.
  %
- %    o exception: return any errors or warnings in this structure.
+ %    o exception: return any errors or warnings in this structure. 
  %
  */
 
@@ -168,21 +165,21 @@ static void SetHeaderFromIPL(Image *image, IPLInfo *ipl){
   image->columns = ipl->width;
   image->rows = ipl->height;
   image->depth = ipl->depth;
-  image->x_resolution = 1;
-  image->y_resolution = 1;
+  image->resolution.x = 1;
+  image->resolution.y = 1;
 }
 
 
 static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-
-  /*
-    Declare variables.
-  */
+  
+  /* 
+  Declare variables 
+   */
   Image *image;
 
   MagickBooleanType status;
-  register PixelPacket *q;
+  Quantum *q;
   unsigned char magick[12], *pixels;
   ssize_t count;
   ssize_t y;
@@ -211,91 +208,94 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
   {
     image=DestroyImageList(image);
     return((Image *) NULL);
   }
-
+  
   /*
-    Read IPL image
-  */
+   Read IPL image
+   */
 
-  /*
-    Determine endianness
-    If we get back "iiii", we have LSB,"mmmm", MSB
-  */
+  /* 
+    Determine endianness 
+   If we get back "iiii", we have LSB,"mmmm", MSB
+   */
   count=ReadBlob(image,4,magick);
   if (count != 4)
     ThrowReaderException(CorruptImageError, "ImproperImageHeader");
-  if((LocaleNCompare((char *) magick,"iiii",4) == 0))
+  if((LocaleNCompare((char *) magick,"iiii",4) == 0))  
     image->endian=LSBEndian;
   else{
-    if((LocaleNCompare((char *) magick,"mmmm",4) == 0))
+    if((LocaleNCompare((char *) magick,"mmmm",4) == 0)) 
       image->endian=MSBEndian;
     else{
       ThrowReaderException(CorruptImageError, "ImproperImageHeader");
     }
   }
   /* Skip o'er the next 8 bytes (garbage) */
-  count=ReadBlob(image, 8, magick);
+  count=ReadBlob(image, 8, magick); 
   /*
    Excellent, now we read the header unimpeded.
    */
-  count=ReadBlob(image,4,magick);
+  count=ReadBlob(image,4,magick); 
   if((count != 4) || (LocaleNCompare((char *) magick,"data",4) != 0))
     ThrowReaderException(CorruptImageError, "ImproperImageHeader");
-  ipl_info.size=ReadBlobLong(image);
-  ipl_info.width=ReadBlobLong(image);
-  ipl_info.height=ReadBlobLong(image);
+  ipl_info.size=ReadBlobLong(image); 
+  ipl_info.width=ReadBlobLong(image); 
+  ipl_info.height=ReadBlobLong(image); 
   if((ipl_info.width == 0UL) || (ipl_info.height == 0UL))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  ipl_info.colors=ReadBlobLong(image);
-  if(ipl_info.colors == 3){ SetImageColorspace(image,sRGBColorspace);}
-  else { image->colorspace = GRAYColorspace; }
-  ipl_info.z=ReadBlobLong(image);
-  ipl_info.time=ReadBlobLong(image);
-
-  ipl_info.byteType=ReadBlobLong(image);
-
+  ipl_info.colors=ReadBlobLong(image); 
+  if (ipl_info.colors == 3)
+    {
+      if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)      
+        SetImageColorspace(image,sRGBColorspace,exception);
+    }
+  else
+    image->colorspace = GRAYColorspace;
+  ipl_info.z=ReadBlobLong(image); 
+  ipl_info.time=ReadBlobLong(image); 
+  ipl_info.byteType=ReadBlobLong(image); 
 
   /* Initialize Quantum Info */
 
   switch (ipl_info.byteType) {
-    case 0:
+    case 0: 
       ipl_info.depth=8;
       quantum_format = UnsignedQuantumFormat;
       break;
-    case 1:
+    case 1: 
       ipl_info.depth=16;
       quantum_format = SignedQuantumFormat;
       break;
-    case 2:
+    case 2: 
       ipl_info.depth=16;
       quantum_format = UnsignedQuantumFormat;
       break;
-    case 3:
+    case 3: 
       ipl_info.depth=32;
       quantum_format = SignedQuantumFormat;
       break;
     case 4: ipl_info.depth=32;
       quantum_format = FloatingPointQuantumFormat;
       break;
-    case 5:
+    case 5: 
       ipl_info.depth=8;
       quantum_format = UnsignedQuantumFormat;
       break;
-    case 6:
+    case 6: 
       ipl_info.depth=16;
       quantum_format = UnsignedQuantumFormat;
       break;
-    case 10:
+    case 10:  
       ipl_info.depth=64;
       quantum_format = FloatingPointQuantumFormat;
-      break;
-    default:
+      break; 
+    default: 
       ipl_info.depth=16;
       quantum_format = UnsignedQuantumFormat;
       break;
@@ -311,10 +311,10 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   /* Thats all we need if we are pinging. */
   if (image_info->ping != MagickFalse)
-  {
-    (void) CloseBlob(image);
-    return(GetFirstImageInList(image));
-  }
+    {
+      (void) CloseBlob(image);
+      return(GetFirstImageInList(image));
+    }
   length=image->columns;
   quantum_type=GetQuantumType(image,exception);
  do
@@ -324,12 +324,9 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    status=SetImageExtent(image,image->columns,image->rows);
+    status=SetImageExtent(image,image->columns,image->rows,exception);
     if (status == MagickFalse)
-      {
-        InheritException(exception,&image->exception);
-        return(DestroyImageList(image));
-      }
+      return(DestroyImageList(image));
 /*
    printf("Length: %.20g, Memory size: %.20g\n", (double) length,(double)
      image->depth);
@@ -340,7 +337,7 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
      status=SetQuantumFormat(image,quantum_info,quantum_format);
      if (status == MagickFalse)
        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-     pixels=GetQuantumPixels(quantum_info);
+     pixels=(unsigned char *) GetQuantumPixels(quantum_info); 
      if(image->columns != ipl_info.width){
 /*
      printf("Columns not set correctly!  Wanted: %.20g, got: %.20g\n",
@@ -348,15 +345,15 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
 */
      }
 
-    /*
-      Covert IPL binary to pixel packets
-    */
-
+    /* 
+    Covert IPL binary to pixel packets
+     */
+    
   if(ipl_info.colors == 1){
       for(y = 0; y < (ssize_t) image->rows; y++){
         (void) ReadBlob(image, length*image->depth/8, pixels);
         q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-        if (q == (PixelPacket *) NULL)
+        if (q == (Quantum *) NULL)
                 break;
         (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
           GrayQuantum,pixels,exception);
@@ -368,17 +365,17 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       for(y = 0; y < (ssize_t) image->rows; y++){
         (void) ReadBlob(image, length*image->depth/8, pixels);
         q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-        if (q == (PixelPacket *) NULL)
+        if (q == (Quantum *) NULL)
                 break;
         (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
-          RedQuantum,pixels,exception);
+          RedQuantum,pixels,exception);  
         if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
       }
       for(y = 0; y < (ssize_t) image->rows; y++){
         (void) ReadBlob(image, length*image->depth/8, pixels);
         q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-        if (q == (PixelPacket *) NULL)
+        if (q == (Quantum *) NULL)
           break;
         (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
           GreenQuantum,pixels,exception);
@@ -388,7 +385,7 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       for(y = 0; y < (ssize_t) image->rows; y++){
         (void) ReadBlob(image, length*image->depth/8, pixels);
         q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-        if (q == (PixelPacket *) NULL)
+        if (q == (Quantum *) NULL)
           break;
         (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
           BlueQuantum,pixels,exception);
@@ -397,7 +394,7 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
    }
    SetQuantumImageType(image,quantum_type);
-
+ 
     t_count++;
   quantum_info = DestroyQuantumInfo(quantum_info);
 
@@ -407,30 +404,30 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
                  image->filename);
       break;
     }
-   if (t_count < ipl_info.z * ipl_info.time)
+   if (t_count < (size_t) (ipl_info.z*ipl_info.time))
      {
       /*
        Proceed to next image.
        */
-      AcquireNextImage(image_info, image);
+      AcquireNextImage(image_info,image,exception);
       if (GetNextImageInList(image) == (Image *) NULL)
         {
           status=MagickFalse;
           break;
         }
-      image=SyncNextImageInList(image);
+      image=SyncNextImageInList(image); 
       status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
         GetBlobSize(image));
       if (status == MagickFalse)
         break;
     }
-  } while (t_count < ipl_info.z*ipl_info.time);
+  } while (t_count < (size_t) (ipl_info.z*ipl_info.time));
   CloseBlob(image);
   if (status == MagickFalse)
     return(DestroyImageList(image));
   return(GetFirstImageInList(image));
 }
-
+
 /*
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %                                                                             %
@@ -443,7 +440,7 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %
  % RegisterIPLImage() add attributes for the Scanalytics IPL image format to the
- % list of supported formats.
+ % list of supported formats.  
  %
  %
  */
@@ -451,16 +448,13 @@ ModuleExport size_t RegisterIPLImage(void)
 {
   MagickInfo
     *entry;
-
-  entry=SetMagickInfo("IPL");
+  
+  entry=AcquireMagickInfo("IPL","IPL","IPL Image Sequence");
   entry->decoder=(DecodeImageHandler *) ReadIPLImage;
   entry->encoder=(EncodeImageHandler *) WriteIPLImage;
   entry->magick=(IsImageFormatHandler *) IsIPL;
-  entry->adjoin=MagickTrue;
-  entry->description=ConstantString("IPL Image Sequence");
-  entry->magick_module=ConstantString("IPL");
-  entry->endian_support=MagickTrue;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags|=CoderEndianSupportFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -488,47 +482,47 @@ ModuleExport void UnregisterIPLImage(void)
 {
   (void) UnregisterMagickInfo("IPL");
 }
-
+
 /*
- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
- %                                                                             %
- %                                                                             %
- %                                                                             %
- %   W r i t e I P L I m a g e                                                 %
- %                                                                             %
- %                                                                             %
- %                                                                             %
- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
- %
- %  WriteIPLImage() writes an image to a file in Scanalytics IPLabimage format.
- %
- %  The format of the WriteIPLImage method is:
- %
- %      MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
- %
- %  A description of each parameter follows.
- %
- %    o image_info: The image info.
- %
- %    o image:  The image.
- %
- */
-
-static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   W r i t e I P L I m a g e                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  WriteIPLImage() writes an image to a file in Scanalytics IPLabimage format.
+%
+%  The format of the WriteIPLImage method is:
+%
+%      MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
+%       Image *image,ExceptionInfo *exception)
+%
+%  A description of each parameter follows.
+%
+%    o image_info: The image info.
+%
+%    o image:  The image.
+%
+%    o exception: return any errors or warnings in this structure.
+%
+*/
+static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image,
+  ExceptionInfo *exception)
 {
-  ExceptionInfo
-    *exception;
-
   IPLInfo
     ipl_info;
 
   MagickBooleanType
     status;
-
+  
   MagickOffsetType
     scene;
-
-  register const PixelPacket
+  
+  const Quantum
     *p;
 
   QuantumInfo
@@ -539,10 +533,10 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
 
   ssize_t
     y;
-
+  
   unsigned char
     *pixels;
-
+ 
    /*
     Open output image file.
   */
@@ -552,19 +546,21 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
   assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickCoreSignature);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
   scene=0;
-
-  quantum_info=AcquireQuantumInfo(image_info, image);
+  
+  quantum_info=AcquireQuantumInfo(image_info,image);
   if (quantum_info == (QuantumInfo *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   if ((quantum_info->format == UndefinedQuantumFormat) &&
-      (IsHighDynamicRangeImage(image,&image->exception) != MagickFalse))
+      (IsHighDynamicRangeImage(image,exception) != MagickFalse))
     SetQuantumFormat(image,quantum_info,FloatingPointQuantumFormat);
   switch(quantum_info->depth){
-  case 8:
+  case 8: 
     ipl_info.byteType = 0;
     break;
   case 16:
@@ -586,10 +582,10 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
   case 64:
     ipl_info.byteType = 10;
     break;
-  default:
-    ipl_info.byteType = 2;
+  default: 
+    ipl_info.byteType = 2; 
     break;
-
+    
   }
   imageListLength=GetImageListLength(image);
   ipl_info.z = (unsigned int) imageListLength;
@@ -597,15 +593,16 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
   ipl_info.time = 1;
   ipl_info.width = (unsigned int) image->columns;
   ipl_info.height = (unsigned int) image->rows;
-  (void) TransformImageColorspace(image,sRGBColorspace);
+  if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+      (void) TransformImageColorspace(image,sRGBColorspace,exception);
   if(IssRGBCompatibleColorspace(image->colorspace) != MagickFalse) { ipl_info.colors = 3; }
   else{ ipl_info.colors = 1; }
-
-  ipl_info.size = (unsigned int) (28 +
+  
+  ipl_info.size = (unsigned int) (28 + 
     ((image->depth)/8)*ipl_info.height*ipl_info.width*ipl_info.colors*ipl_info.z);
-
+  
   /* Ok!  Calculations are done.  Lets write this puppy down! */
-
+  
   /*
     Write IPL header.
   */
@@ -623,7 +620,7 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlob(image, 4, (const unsigned char *) "100f");
   (void) WriteBlob(image, 4, (const unsigned char *) "data");
   (void) WriteBlobLong(image, ipl_info.size);
-  (void) WriteBlobLong(image, ipl_info.width);
+  (void) WriteBlobLong(image, ipl_info.width); 
   (void) WriteBlobLong(image, ipl_info.height);
   (void) WriteBlobLong(image, ipl_info.colors);
   if(image_info->adjoin == MagickFalse)
@@ -632,22 +629,21 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobLong(image, ipl_info.z);
   (void) WriteBlobLong(image, ipl_info.time);
   (void) WriteBlobLong(image, ipl_info.byteType);
-
-  exception=(&image->exception);
+  
   do
     {
       /*
   Convert MIFF to IPL raster pixels.
       */
-      pixels=GetQuantumPixels(quantum_info);
+      pixels=(unsigned char *) GetQuantumPixels(quantum_info);
   if(ipl_info.colors == 1){
   /* Red frame */
   for(y = 0; y < (ssize_t) ipl_info.height; y++){
-    p=GetAuthenticPixels(image,0,y,image->columns,1,exception);
-    if (p == (PixelPacket *) NULL)
+    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
+    if (p == (const Quantum *) NULL)
       break;
-    (void) ExportQuantumPixels(image,(const CacheView *) NULL, quantum_info,
-      GrayQuantum, pixels,&image->exception);
+    (void) ExportQuantumPixels(image,(CacheView *) NULL, quantum_info,
+      GrayQuantum, pixels,exception);
     (void) WriteBlob(image, image->columns*image->depth/8, pixels);
   }
 
@@ -655,30 +651,30 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
   if(ipl_info.colors == 3){
   /* Red frame */
   for(y = 0; y < (ssize_t) ipl_info.height; y++){
-    p=GetAuthenticPixels(image,0,y,image->columns,1,exception);
-    if (p == (PixelPacket *) NULL)
+    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
+    if (p == (const Quantum *) NULL)
       break;
-    (void) ExportQuantumPixels(image,(const CacheView *) NULL, quantum_info,
-      RedQuantum, pixels,&image->exception);
+    (void) ExportQuantumPixels(image,(CacheView *) NULL, quantum_info,
+      RedQuantum, pixels,exception);
     (void) WriteBlob(image, image->columns*image->depth/8, pixels);
   }
 
     /* Green frame */
     for(y = 0; y < (ssize_t) ipl_info.height; y++){
-      p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
-      if (p == (PixelPacket *) NULL)
+      p=GetVirtualPixels(image,0,y,image->columns,1,exception);
+      if (p == (const Quantum *) NULL)
         break;
-      (void) ExportQuantumPixels(image,(const CacheView *) NULL, quantum_info,
-        GreenQuantum, pixels,&image->exception);
+      (void) ExportQuantumPixels(image,(CacheView *) NULL, quantum_info,
+        GreenQuantum, pixels,exception);
       (void) WriteBlob(image, image->columns*image->depth/8, pixels);
     }
     /* Blue frame */
     for(y = 0; y < (ssize_t) ipl_info.height; y++){
-      p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
-      if (p == (PixelPacket *) NULL)
+      p=GetVirtualPixels(image,0,y,image->columns,1,exception);
+      if (p == (const Quantum *) NULL)
         break;
-      (void) ExportQuantumPixels(image,(const CacheView *) NULL, quantum_info,
-        BlueQuantum, pixels,&image->exception);
+      (void) ExportQuantumPixels(image,(CacheView *) NULL, quantum_info,
+        BlueQuantum, pixels,exception);
       (void) WriteBlob(image, image->columns*image->depth/8, pixels);
       if (image->previous == (Image *) NULL)
         {
